@@ -18,7 +18,6 @@ import select
 
 
 from MAVProxy.modules.lib import textconsole
-from MAVProxy.modules.lib import mp_settings
 from MAVProxy.modules.lib import rline
 from MAVProxy.modules.lib import mp_module
 from MAVProxy.modules.lib import dumpstacks
@@ -89,10 +88,17 @@ def say(text, priority='important'):
     '''text and/or speech output'''
     mpstate.functions.say(text, priority)
 
+def add_input(cmd, immediate=False):
+    '''add some command input to be processed'''
+    if immediate:
+        process_stdin(cmd)
+    else:
+        mpstate.input_queue.put(cmd)
+
 class MAVFunctions(object):
     '''core functions available in modules'''
     def __init__(self):
-        self.process_stdin = process_stdin
+        self.process_stdin = add_input
         self.param_set = param_set
         self.get_mav_param = get_mav_param
         self.say = say_text
@@ -105,26 +111,28 @@ class MPState(object):
         self.map_functions = {}
         self.vehicle_type = None
         self.vehicle_name = None
-        self.settings = mp_settings.MPSettings(
-            [ ('link', int, 1),
-              ('altreadout', int, 10),
-              ('distreadout', int, 200),
-              ('heartbeat', int, 1),
-              ('numcells', int, 1),
-              ('mavfwd', int, 1),
-              ('mavfwd_rate', int, 0),
-              ('streamrate', int, 4),
-              ('streamrate2', int, 4),
-              ('heartbeatreport', int, 1),
-              ('moddebug', int, 0),
-              ('rc1mul', int, 1),
-              ('rc2mul', int, 1),
-              ('rc4mul', int, 1),
-              ('shownoise', int, 1),
-              ('basealt', int, 0),
-              ('wpalt', int, 100),
-              ('flushlogs', int, 0),
-              ('requireexit', int, 0)]
+        from MAVProxy.modules.lib.mp_settings import MPSettings, MPSetting
+        self.settings = MPSettings(
+            [ MPSetting('link', int, 1, 'Primary Link', tab='Link', range=(0,4), increment=1),
+              MPSetting('streamrate', int, 4, 'Stream rate link1', range=(0,20), increment=1),
+              MPSetting('streamrate2', int, 4, 'Stream rate link2', range=(0,20), increment=1),
+              MPSetting('heartbeat', int, 1, 'Heartbeat rate', range=(0,5), increment=1),
+              MPSetting('mavfwd', bool, True, 'Allow forwarded control'),
+              MPSetting('mavfwd_rate', bool, False, 'Allow forwarded rate control'),
+              MPSetting('shownoise', bool, True, 'Show non-MAVLink data'),
+              
+              MPSetting('altreadout', int, 10, 'Altitude Readout',
+                        range=(0,100), increment=1, tab='Announcements'),
+              MPSetting('distreadout', int, 200, 'Distance Readout', range=(0,10000), increment=1),
+
+              MPSetting('moddebug', int, 0, 'Module Debug Level', range=(0,3), increment=1, tab='Debug'),
+              MPSetting('numcells', int, 1, range=(0,10), increment=1),
+              MPSetting('flushlogs', bool, False, 'Flush logs on every packet'),
+              MPSetting('requireexit', bool, False, 'Require exit command'),
+
+              MPSetting('basealt', int, 0, 'Base Altitude', range=(0,30000), increment=1, tab='Altitude'),
+              MPSetting('wpalt', int, 100, 'Default WP Altitude', range=(0,10000), increment=1),
+              MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1)]
             )
 
         self.completions = {
@@ -184,10 +192,6 @@ def param_set(name, value, retries=3):
     name = name.upper()
     return mpstate.mav_param.mavset(mpstate.master(), name, value, retries=retries)
 
-def cmd_reboot(args):
-    '''reboot autopilot'''
-    mpstate.master().reboot_autopilot()
-
 def cmd_script(args):
     '''run a script'''
     if len(args) < 1:
@@ -208,33 +212,6 @@ def cmd_status(args):
         for pattern in args:
             mpstate.status.show(sys.stdout, pattern=pattern)
 
-def cmd_bat(args):
-    '''show battery levels'''
-    print("Flight battery:   %u%%" % mpstate.status.battery_level)
-    print("Avionics battery: %u%%" % mpstate.status.avionics_battery_level)
-
-def cmd_alt(args):
-    '''show altitude'''
-    print("Altitude:  %.1f" % mpstate.status.altitude)
-
-
-def cmd_up(args):
-    '''adjust TRIM_PITCH_CD up by 5 degrees'''
-    if len(args) == 0:
-        adjust = 5.0
-    else:
-        adjust = float(args[0])
-    old_trim = get_mav_param('TRIM_PITCH_CD', None)
-    if old_trim is None:
-        print("Existing trim value unknown!")
-        return
-    new_trim = int(old_trim + (adjust*100))
-    if math.fabs(new_trim - old_trim) > 1000:
-        print("Adjustment by %d too large (from %d to %d)" % (adjust*100, old_trim, new_trim))
-        return
-    print("Adjusting TRIM_PITCH_CD from %d to %d" % (old_trim, new_trim))
-    param_set('TRIM_PITCH_CD', new_trim)
-
 def cmd_setup(args):
     mpstate.status.setup_mode = True
     mpstate.rl.set_prompt("")
@@ -285,11 +262,11 @@ def load_module(modname, quiet=False):
             else:
                 ex = "%s.init did not return a MPModule instance" % modname
                 break
-        except ImportError, msg:
+        except ImportError as msg:
             ex = msg
             if mpstate.settings.moddebug > 1:
                 import traceback
-                print traceback.format_exc()
+                print(traceback.format_exc())
     print("Failed to load module: %s" % ex)
     return False
 
@@ -375,15 +352,6 @@ def cmd_alias(args):
         print(usage)
         return
 
-def cmd_time(args):
-  '''show autopilot time'''
-  tusec = mpstate.master().field('SYSTEM_TIME', 'time_unix_usec', 0)
-  if tusec == 0:
-      print("No SYSTEM_TIME time available")
-      return
-  print("%s (%s)\n" % (time.ctime(tusec * 1.0e-6), time.ctime()))
-
-
 # http://stackoverflow.com/questions/211100/pythons-import-doesnt-work-as-expected
 # has info on why this is necessary.
 
@@ -403,15 +371,10 @@ command_map = {
     'reset'   : (cmd_reset,    'reopen the connection to the MAVLink master'),
     'status'  : (cmd_status,   'show status'),
     'set'     : (cmd_set,      'mavproxy settings'),
-    'bat'     : (cmd_bat,      'show battery levels'),
-    'alt'     : (cmd_alt,      'show relative altitude'),
     'link'    : (cmd_link,     'show link status'),
-    'reboot'  : (cmd_reboot,   'reboot the autopilot'),
-    'up'      : (cmd_up,       'adjust TRIM_PITCH_CD up by 5 degrees'),
     'watch'   : (cmd_watch,    'watch a MAVLink pattern'),
     'module'  : (cmd_module,   'module commands'),
-    'alias'   : (cmd_alias,    'command aliases'),
-    'time'    : (cmd_time,     'Show autopilot time'),
+    'alias'   : (cmd_alias,    'command aliases')
     }
 
 def process_stdin(line):
@@ -700,7 +663,7 @@ def master_callback(m, master):
             have_gps_fix = True
         if 'GPS_RAW_INT' in mpstate.status.msgs and mpstate.status.msgs['GPS_RAW_INT'].fix_type == 3:
             have_gps_fix = True
-        if have_gps_fix and not mpstate.status.have_gps_lock:
+        if have_gps_fix and not mpstate.status.have_gps_lock and m.alt != 0:
                 say("GPS lock at %u meters" % m.alt, priority='notification')
                 mpstate.status.have_gps_lock = True
 
@@ -773,7 +736,7 @@ def master_callback(m, master):
                 continue
             try:
                 mod.mavlink_packet(m)
-            except Exception, msg:
+            except Exception as msg:
                 if mpstate.settings.moddebug == 1:
                     print(msg)
                 elif mpstate.settings.moddebug > 1:
@@ -869,7 +832,11 @@ def open_logs():
         mode = 'w'
     logfile = opts.logfile
     if opts.aircraft is not None:
-        dirname = "%s/logs/%s" % (opts.aircraft, time.strftime("%Y-%m-%d"))
+        if opts.mission is not None:
+            print(opts.mission)
+            dirname = "%s/logs/%s/Mission%s" % (opts.aircraft, time.strftime("%Y-%m-%d"), opts.mission)
+        else:
+            dirname = "%s/logs/%s" % (opts.aircraft, time.strftime("%Y-%m-%d"))
         mkdir_p(dirname)
         highest = None
         for i in range(1, 10000):
@@ -966,7 +933,7 @@ def periodic_tasks():
         if hasattr(m, 'idle_task'):
             try:
                 m.idle_task()
-            except Exception, msg:
+            except Exception as msg:
                 if mpstate.settings.moddebug == 1:
                     print(msg)
                 elif mpstate.settings.moddebug > 1:
@@ -986,11 +953,11 @@ def main_loop():
     while True:
         if mpstate is None or mpstate.status.exit:
             return
-        if mpstate.rl.line is not None:
-            cmds = mpstate.rl.line.split(';')
+        while not mpstate.input_queue.empty():
+            line = mpstate.input_queue.get()
+            cmds = line.split(';')
             for c in cmds:
                 process_stdin(c)
-            mpstate.rl.line = None
 
         for master in mpstate.mav_master:
             if master.fd is None:
@@ -1036,7 +1003,7 @@ def main_loop():
                     # call the registered read function
                     (fn, args) = mpstate.select_extra[fd]
                     fn(args)
-                except Exception, msg:
+                except Exception as msg:
                     if mpstate.settings.moddebug == 1:
                         print(msg)
                     # on an exception, remove it from the select list
@@ -1047,15 +1014,13 @@ def main_loop():
 def input_loop():
     '''wait for user input'''
     while mpstate.status.exit != True:
-        while mpstate.rl.line is not None:
-            time.sleep(0.01)
         try:
             if mpstate.status.exit != True:
                 line = raw_input(mpstate.rl.prompt)
         except EOFError:
             mpstate.status.exit = True
             sys.exit(1)
-        mpstate.rl.line = line
+        mpstate.input_queue.put(line)
 
 
 def run_script(scriptfile):
@@ -1128,9 +1093,14 @@ if __name__ == '__main__':
     parser.add_option("--continue", dest='continue_mode', action='store_true', default=False, help="continue logs")
     parser.add_option("--dialect",  default="ardupilotmega", help="MAVLink dialect")
     parser.add_option("--rtscts",  action='store_true', help="enable hardware RTS/CTS flow control")
+    parser.add_option("--mission", dest="mission", help="mission name", default=None)
 
     (opts, args) = parser.parse_args()
 
+    # warn people about ModemManager which interferes badly with APM and Pixhawk
+    if os.path.exists("/usr/sbin/ModemManager"):
+        print("WARNING: You should uninstall ModemManager as it conflicts with APM and Pixhawk")
+
     if opts.mav09:
         os.environ['MAVLINK09'] = '1'
     from pymavlink import mavutil, mavparm
@@ -1217,6 +1187,7 @@ Auto-detected serial ports are:
     battery_period = mavutil.periodic_event(0.1)
     heartbeat_check_period = mavutil.periodic_event(0.33)
 
+    mpstate.input_queue = Queue.Queue()
     mpstate.rl = rline.rline("MAV> ", mpstate)
     if opts.setup:
         mpstate.rl.set_prompt("")
@@ -1235,8 +1206,8 @@ Auto-detected serial ports are:
 
     if not opts.setup:
         # some core functionality is in modules
-        standard_modules = ['log','rally','fence','param','relay',
-                            'tuneopt','arm','mode','calibration','rc','wp','auxopt']
+        standard_modules = ['log', 'wp', 'rally','fence','param','relay',
+                            'tuneopt','arm','mode','calibration','rc','auxopt','misc']
         for m in standard_modules:
             load_module(m, quiet=True)
 
@@ -1288,7 +1259,7 @@ Auto-detected serial ports are:
     #this loop executes after leaving the above loop and is for cleanup on exit
     for (m,pm) in mpstate.modules:
         if hasattr(m, 'unload'):
-            print "Unloading module:", m.name
+            print("Unloading module %s" % m.name)
             m.unload()
         
     sys.exit(1)
